@@ -420,7 +420,7 @@ class BinaryClassificationMetrics:
         metrics_df = pd.DataFrame(
             data=self.metrics_dict_,
             index=[
-                f"""Threshold = {self.threshold:.3f} | Average =
+                f"""Threshold = {self.threshold:.{self.precision_digits}f} | Average =
                 {self.average_method.title()}"""
             ],
         )
@@ -536,7 +536,7 @@ class BinaryClassificationMetrics:
         """
         return ["binary", "weighted", "macro", "micro"]
 
-    def plot(self, figsize=None):
+    def plot(self, figsize=None, save_path=None):
         """Function to plot binary classification metrics.
         This function is a helper function based on the plotting_dict
         attribute of the BinaryClassificationMetrics class.
@@ -545,9 +545,15 @@ class BinaryClassificationMetrics:
         ----------
         figsize: tuple, optional, (default=(12, 12))
             Figure size
+
+        save_path: str, optional (default=None)
+            The full or relative path to save the plot including the image format.
+            For example "myplot.png" or "../../myplot.pdf"
         """
 
-        plot_binary_classification_metrics(figsize, **self.plotting_dict_)
+        plot_binary_classification_metrics(
+            figsize=figsize, save_path=save_path, **self.plotting_dict_
+        )
 
 
 class RegressionMetrics:
@@ -894,7 +900,7 @@ class RegressionMetrics:
 
         return plotting_dict
 
-    def plot(self, figsize=None):
+    def plot(self, figsize=None, save_path=None):
         """
         Function to plot binary classification metrics.
         This function is a helper function based on the plotting_dict
@@ -904,6 +910,12 @@ class RegressionMetrics:
         ----------
         figsize: tuple, optional, (default=(12, 12))
             Figure size
+
+        save_path: str, optional (default=None)
+            The full or relative path to save the plot including the image format.
+            For example "myplot.png" or "../../myplot.pdf"
         """
 
-        plot_regression_metrics(figsize, **self.plotting_dict_)
+        plot_regression_metrics(
+            figsize=figsize, save_path=save_path, **self.plotting_dict_
+        )
