@@ -17,7 +17,7 @@ warnings.filterwarnings("ignore")
 display(HTML("<style>.container { width:95% !important; }</style>"))
 
 
-def plot_binary_classification_metrics(figsize=None, **kwargs):
+def plot_binary_classification_metrics(figsize=None, save_path=None, **kwargs):
     """
     Function to plot binary classification metrics.
     This function is a helper function based on the plotting_dict
@@ -28,6 +28,10 @@ def plot_binary_classification_metrics(figsize=None, **kwargs):
     figsize: tuple, optional, (default=(12, 12))
         Figure size
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -230,6 +234,9 @@ def plot_binary_classification_metrics(figsize=None, **kwargs):
             verticalalignment="bottom",
         )
 
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
@@ -244,6 +251,7 @@ def plot_xfs_feature_frequency(
     markerfacecolor=None,
     markeredgewidth=None,
     fontsize=None,
+    save_path=None,
 ):
     """Function to plot selected features frequency.
     This function is a helper function based on the features_frequency
@@ -283,6 +291,10 @@ def plot_xfs_feature_frequency(
     fontsize: int or float, optional, (default=12)
         Fontsize for xlabel and ylabel, and ticks parameters
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -375,11 +387,21 @@ def plot_xfs_feature_frequency(
     ax.set_ylabel("Feature", fontsize=fontsize)
     ax.set_title("Important Features Frequency", fontsize=fontsize)
     ax.tick_params(axis="both", which="major", labelsize=fontsize)
+
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
 def plot_xfs_cv_results(
-    figsize=None, int_color=None, ext_color=None, sharex=False, sharey=False, **kwargs
+    figsize=None,
+    int_color=None,
+    ext_color=None,
+    sharex=False,
+    sharey=False,
+    save_path=None,
+    **kwargs,
 ):
     """Function to plot the cross-validation results ofXGBoostFeatureSelector.
     It visualizes the internal and external performance during the
@@ -405,6 +427,10 @@ def plot_xfs_cv_results(
     sharey: bool, optional, (default=False)
         Flag to share "Y" axis for each row of subplots
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     kwargs: dict()
         Plotting object plotting_cv_
 
@@ -450,6 +476,10 @@ def plot_xfs_cv_results(
     ax4.set(
         title=f"External {kwargs['n_splits']}-Folds CV {kwargs['eval_metric']} - Test"
     )
+
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
@@ -463,6 +493,7 @@ def plot_xgb_cv_results(
     train_std_color=None,
     test_color=None,
     test_std_color=None,
+    save_path=None,
 ):
     """Function to plot cv results of XGBoostCVClassifier.
 
@@ -496,6 +527,10 @@ def plot_xgb_cv_results(
     test_std_color: str, optional, (default="#D0AAF3")
         Color of the edge color of the testing std bars
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -590,6 +625,9 @@ def plot_xgb_cv_results(
     ax.tick_params(axis="both", which="major", labelsize=12)
     ax.legend(loc=0, prop={"size": 12}, framealpha=0.0)
 
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
@@ -603,6 +641,7 @@ def plot_xgb_feature_importance(
     markerfacecolor=None,
     markeredgewidth=None,
     fontsize=None,
+    save_path=None,
 ):
     """Function to plot XGBoost feature importance.
     This function is a helper function based on the feature_importance_
@@ -639,6 +678,10 @@ def plot_xgb_feature_importance(
     fontsize: int or float, optional, (default=12)
         Fontsize for xlabel and ylabel, and ticks parameters
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -739,6 +782,10 @@ def plot_xgb_feature_importance(
     ax.set_title("Feature Importance", fontsize=fontsize)
     ax.set(xlim=[None, feature_importance[colx].max() * 1.13])
     ax.tick_params(axis="both", which="major", labelsize=fontsize)
+
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
@@ -758,6 +805,7 @@ def plot_shap_summary(
     class_names=None,
     class_inds=None,
     color_bar_label=None,
+    save_path=None,
 ):
     """Function to plot shap summary plot.
     This function is a helper function to plot the shap summary plot
@@ -816,6 +864,10 @@ def plot_shap_summary(
     color_bar_label: str, optional, (default="Feature Value")
         Label for color bar
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -871,6 +923,10 @@ def plot_shap_summary(
         class_inds=class_inds,
         color_bar_label=color_bar_label,
     )
+
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
@@ -889,6 +945,7 @@ def plot_shap_waterfall(
     max_display=None,
     title=None,
     fontsize=None,
+    save_path=None,
 ):
     """Function to plot shap summary plot.
     This function is a helper function to plot the shap summary plot
@@ -943,6 +1000,10 @@ def plot_shap_waterfall(
     fontsize: int or float, optional, (default=12)
         Fontsize for xlabel and ylabel, and ticks parameters
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -1083,10 +1144,13 @@ def plot_shap_waterfall(
     )
     ax2.set(xlabel="Composition Ratio (%)")
 
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
-def plot_regression_metrics(figsize=None, **kwargs):
+def plot_regression_metrics(figsize=None, save_path=None, **kwargs):
     """Function to plot regression metrics.
     This function is a helper function based on the plotting_dict
     attribute of the RegressionMetrics class.
@@ -1096,6 +1160,10 @@ def plot_regression_metrics(figsize=None, **kwargs):
     figsize: tuple, optional, (default=(12, 12))
         Figure size
 
+    save_path: str, optional (default=None)
+        The full or relative path to save the plot including the image format.
+        For example "myplot.png" or "../../myplot.pdf"
+
     Returns None
     """
 
@@ -1271,6 +1339,9 @@ def plot_regression_metrics(figsize=None, **kwargs):
     ax6.tick_params(axis="both", which="major", labelsize=12)
     ax6.legend(prop={"size": 12}, loc=4, framealpha=0.0)
 
+    if save_path:
+        plt.savefig(save_path, bbox_inches="tight", dpi=200)
+
     plt.show()
 
 
@@ -1319,7 +1390,7 @@ def plot_glmnet_cv_results(
         tick_params, and legend are resized with 0.85, 0.85, 0.75,
         and 0.85 fraction of title fontsize, respectively.
 
-    grid : bool, optional (default=True)
+    grid: bool, optional (default=True)
         Whether to show (x,y) grid on the plot.
 
     legend: bool, optional (default=True)
